"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");
Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.combineMiddlewareWithActionHandlers = combineMiddlewareWithActionHandlers;
exports["default"] = composeMiddleware;
var _lodash = require("lodash");
var _createMiddleware = _interopRequireDefault(require("./createMiddleware.js"));
var _checkForNonFunctions = _interopRequireDefault(require("./checkForNonFunctions.js"));
var _addCreationStack = _interopRequireDefault(require("./addCreationStack.js"));
/**
 * @param middlewares an array of one or more middlewares.
 * @returns a middleware that calls all of the given middlewares in sequence.
 */
function naiveComposeMiddleware(middlewares) {
  if (middlewares.length === 0) return function (store) {
    return function (dispatch) {
      return dispatch;
    };
  };
  if (middlewares.length === 1) return middlewares[0];
  return function (store) {
    return function (next) {
      var handleAction = middlewares.reduceRight(function (next, handler) {
        return handler(store)(next);
      }, next);
      if (process.env.NODE_ENV !== 'production') handleAction = (0, _addCreationStack["default"])(handleAction, 'middleware');
      return handleAction;
    };
  };
}
function composeActionHandlers(actionHandlers) {
  return (0, _createMiddleware["default"])((0, _lodash.mapValues)(actionHandlers, naiveComposeMiddleware));
}
function combineMiddlewareWithActionHandlers() {
  for (var _len = arguments.length, middlewares = new Array(_len), _key = 0; _key < _len; _key++) {
    middlewares[_key] = arguments[_key];
  }
  if (process.env.NODE_ENV !== 'production') (0, _checkForNonFunctions["default"])(middlewares, 'middlewares');
  var optimizedMiddlewares = [];
  var actionHandlers = {};
  middlewares.forEach(function (middleware) {
    if (middleware.actionHandlers) {
      (0, _lodash.forEach)(middleware.actionHandlers, function (actionHandler, type) {
        ;
        (actionHandlers[type] || (actionHandlers[type] = [])).push(actionHandler);
      });
    } else {
      if ((0, _lodash.size)(actionHandlers)) {
        optimizedMiddlewares.push(composeActionHandlers(actionHandlers));
        actionHandlers = {};
      }
      optimizedMiddlewares.push(middleware);
    }
  });
  if ((0, _lodash.size)(actionHandlers)) optimizedMiddlewares.push(composeActionHandlers(actionHandlers));
  return optimizedMiddlewares;
}
function composeMiddleware() {
  return naiveComposeMiddleware(combineMiddlewareWithActionHandlers.apply(void 0, arguments));
}