"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");
Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.combineReducersWithActionHandlers = combineReducersWithActionHandlers;
exports["default"] = composeReducers;
var _lodash = require("lodash");
var _createReducer = _interopRequireDefault(require("./createReducer.js"));
var _addCreationStack = _interopRequireDefault(require("./addCreationStack.js"));
var _checkForNonFunctions = _interopRequireDefault(require("./checkForNonFunctions.js"));
/**
 * @param reducers an array of one or more reducers.
 * @returns a reducer that calls all of the given reducers in sequence.
 */
function naiveComposeReducers(reducers) {
  if (reducers.length === 0) return function (state) {
    return state;
  };
  if (reducers.length === 1) return reducers[0];
  var initialState = reducers.reduce(function (state, reducer) {
    return reducer(state, {
      type: ''
    });
  }, undefined);
  return function () {
    var state = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : initialState;
    var action = arguments.length > 1 ? arguments[1] : undefined;
    return reducers.reduce(function (state, reducer) {
      return reducer(state, action);
    }, state);
  };
}
function composeActionHandlers(initialState, actionHandlers) {
  return (0, _createReducer["default"])(initialState, (0, _lodash.mapValues)(actionHandlers, naiveComposeReducers));
}
function combineReducersWithActionHandlers() {
  for (var _len = arguments.length, reducers = new Array(_len), _key = 0; _key < _len; _key++) {
    reducers[_key] = arguments[_key];
  }
  if (process.env.NODE_ENV !== 'production') (0, _checkForNonFunctions["default"])(reducers, 'reducers');
  var optimizedReducers = [];
  var initialState = undefined;
  var actionHandlers = {};
  reducers.forEach(function (reducer) {
    if (reducer.actionHandlers) {
      initialState = reducer(initialState, {
        type: ''
      });
      (0, _lodash.forEach)(reducer.actionHandlers, function (actionHandler, type) {
        ;
        (actionHandlers[type] || (actionHandlers[type] = [])).push(actionHandler);
      });
    } else {
      if ((0, _lodash.size)(actionHandlers)) {
        optimizedReducers.push(composeActionHandlers(initialState, actionHandlers));
        actionHandlers = {};
        initialState = undefined;
      }
      optimizedReducers.push(reducer);
    }
  });
  if ((0, _lodash.size)(actionHandlers)) optimizedReducers.push(composeActionHandlers(initialState, actionHandlers));
  return optimizedReducers;
}
function composeReducers() {
  var result = naiveComposeReducers(combineReducersWithActionHandlers.apply(void 0, arguments));
  if (process.env.NODE_ENV !== 'production') result = (0, _addCreationStack["default"])(result, 'reducer');
  return result;
}