"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");
Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = createPluggableMiddleware;
var _lodash = require("lodash");
var _checkForNonFunctions = _interopRequireDefault(require("./checkForNonFunctions.js"));
/**
 * This is used for hot reloading redux middleware.
 *
 * let hmrMiddleware = createPluggableMiddleware(myMiddleware);
 * // hmrMiddleware can be used with applyMiddleware() just like anything else.
 * // but later if you need to replace it you can do:
 * hmrMiddleware.replaceMiddleware(myNewMiddleware);
 */
function createPluggableMiddleware(middleware) {
  if (process.env.NODE_ENV !== 'production') (0, _checkForNonFunctions["default"])(middleware, 'middleware');
  var result = function result(store) {
    return function (next) {
      var dispatch = (0, _lodash.memoize)(function (middleware) {
        return middleware ? middleware(store)(next) : next;
      });
      return function (action) {
        return dispatch(middleware)(action);
      };
    };
  };
  result.replaceMiddleware = function (nextMiddleware) {
    if (process.env.NODE_ENV !== 'production') (0, _checkForNonFunctions["default"])(nextMiddleware, 'nextMiddleware');
    middleware = nextMiddleware;
  };
  return result;
}
module.exports = exports.default;